﻿using System.Text.Json;
using DAL;
using DAS;
using Shared.DTO.Envios_DGA;
using Shared.DTO.VariablesEntorno;
using Shared.Helper;

namespace BLL.Recuperacion_DGA
{
    public class EnvioDGA
    {
        private readonly MedicionScadaRepository _dGAMedicionScadaRepository;
        private readonly RegistrarMedicion _registrarMedicion;

        public EnvioDGA(MedicionScadaRepository dGAMedicionScadaRepository, RegistrarMedicion registrarMedicion)
        {
            _dGAMedicionScadaRepository = dGAMedicionScadaRepository;
            _registrarMedicion = registrarMedicion;
        }

        public async Task<bool> RegistrarMedicionesAsync()
        {
            try
            {
                await WriteLineAndLog("INICIO", "Inicio proceso de recuperación DGA", "");
                WriteLineAndLog("Obteniendo Mediciones Scada", ConsoleColor.Green);
                var mediciones = await _dGAMedicionScadaRepository.ObtenerMedicionesAsync();
                var listaMediciones = new List<Object>();
                foreach (var medicion in mediciones)
                {
                    try
                    {
                        if (!string.IsNullOrEmpty(medicion.Code))
                        {
                            var rutEmpresa = string.Empty;
                            if (medicion.tipo_empresa == null)
                            {
                                continue;
                            }
                            if (medicion.tipo_empresa == "AV")
                            {
                                rutEmpresa = CredencialDGA.RutAv;
                            }
                            if (medicion.tipo_empresa == "EV")
                            {
                                rutEmpresa = CredencialDGA.RutEsval;
                            }

                            List<string> vacios = new List<string>();
                            if (medicion.Caudal == null || medicion.Caudal.Equals(""))
                            {
                                vacios.Add("caudal");
                            }

                            if (medicion.DateOrigen == null)
                            {
                                vacios.Add("fecha origen");
                            }

                            if (medicion.nivelFreaticoDelPozo == null || medicion.nivelFreaticoDelPozo.ToString().Equals(""))
                            {
                                vacios.Add("nivel freatico");
                            }

                            if (medicion.Totalizador == null || medicion.Totalizador.Equals(""))
                            {
                                vacios.Add("totalizador");
                            }

                            if (vacios.Count > 0)
                            {
                                await FileLoggerHelper.InsertarLogsAsync("REGISTRAR", $"Medicion {medicion.Code} no registra {string.Join(", ", vacios)}","");
                            }
                            var fechaEnvio = DateTime.UtcNow;

                            var body = new MedicionSubterraneaRequest
                            {
                                Autenticacion = new Autenticacion
                                {
                                    Password = CredencialDGA.Password,
                                    RutEmpresa = rutEmpresa,
                                    RutUsuario = CredencialDGA.RutUsuario
                                },
                                MedicionSubterranea = new Medicion
                                {
                                    Caudal = medicion.Caudal.ToString() ?? "",
                                    FechaMedicion = medicion.DateOrigen?.ToString("yyyy-MM-dd") ?? "",
                                    HoraMedicion = medicion.DateOrigen?.ToString("HH:mm:ss") ?? "",
                                    NivelFreaticoDelPozo = medicion.nivelFreaticoDelPozo?.ToString() ?? "",
                                    Totalizador = medicion.Totalizador.ToString() ?? "",
                                }
                            };
                            await _registrarMedicion.EnviarMedicionAsync(medicion, body, fechaEnvio);
                            listaMediciones.Add(new {Id = medicion.Id,FechaEnvio = fechaEnvio.ToString("yyyy-MM-dd HH:mm:ss"), Enviado = medicion.Enviado + 1});
                        }

                    }
                    catch (Exception ex)
                    {
                        FileLoggerHelper.LogError($"[Error] {ex.Message}.", ex);
                        WriteLineAndLog($"Error al enviar la medición con ID {medicion.Code}.", ConsoleColor.Red);
                    }
                }
                var listaMedicionesJson = JsonSerializer.Serialize(listaMediciones);

                if (listaMediciones.Count > 0)
                {
                    await MedicionScadaRepository.ActualizarMedicionesAsync(listaMedicionesJson);
                }
                await WriteLineAndLog("FIN","Fin proceso de recuperación DGA","");
            }
            catch (Exception ex)
            {
                FileLoggerHelper.LogError($"[Error] {ex.Message}.", ex);
                WriteLineAndLog($"Error al procesar las mediciones.", ConsoleColor.Red);
                return false;
            }
            return true;
        }

        static void WriteLineAndLog(string msj, ConsoleColor? color = null)
        {
            if (color.HasValue && Enum.IsDefined(typeof(ConsoleColor), color.Value))
            {
                Console.ForegroundColor = (ConsoleColor)color;
                Console.WriteLine(msj);
                Console.ResetColor();
            }
            else
            {
                Console.WriteLine($"{msj}");
            }

            FileLoggerHelper.LogInformation($"{msj}");
        }

        static async Task WriteLineAndLog(string evento, string proceso, string operacion = null, ConsoleColor? color = null)
        {
            if (color.HasValue && Enum.IsDefined(typeof(ConsoleColor), color.Value))
            {
                Console.ForegroundColor = (ConsoleColor)color;
                Console.WriteLine($"{proceso}");
                Console.ResetColor();
            }
            else
            {
                Console.WriteLine($"{proceso}");
            }
            FileLoggerHelper.LogInformation($"{proceso}");
            await FileLoggerHelper.InsertarLogsAsync(evento,proceso,operacion);
        }
    }
}
